// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>
#include <i.h>
#include <util.h>
#include "../../server.h"
#include "../../sim.h"
#include "fit.h"
#include "patch.h"

static int unused __attribute__((unused));

static char config_files[3000];

void fit_build_job(struct server *myserver,char *name)
{
#ifndef windows
char tempsim[1000];
char tempexp[1000];
char command[2000];
char patch_file_path[1000];
	//printf(">>>%s<<<<\n",name);
	sprintf(tempsim,"./sim/%s",name);
	sprintf(tempexp,"./exp/%s",name);
	mkdir(tempsim, S_IRWXU | S_IRWXG | S_IROTH | S_IXOTH);

	strcat(tempexp,"/");
	strcat(tempsim,"/");

	sprintf(command,"cp %s %s",config_files,tempsim);
	unused=system(command);
	//if (get_dump_status(dump_iodump)==TRUE) printf("copy command = %s\n",command);
	sprintf(patch_file_path,"%s/patch.inp",tempexp);
	patch(tempsim,patch_file_path);

	server_add_job(myserver,tempsim,tempsim);
#endif
}

double fit_run_sims(struct fitvars *fitconfig,struct server *myserver)
{
double error=0.0;
#ifndef windows
if (get_dump_status(dump_iodump)==TRUE) printf("fit_run_sims\n");
char tempsim[1000];
char tempexp[1000];
char command[2000];
char tempstring[1000];
double voltage;
int i;

  DIR           *d;
  struct dirent *dir;
  d = opendir(".");
strcpy(config_files,"");
  if (d)
  {
    while ((dir = readdir(d)) != NULL)
    {
     if (strcmp_end(dir->d_name,".inp")==0)
	{
		strcat(config_files,dir->d_name);
		strcat(config_files," ");
	}
    }

    closedir(d);
  }
//printf("copy files= %s\n",config_files);

strcpy(config_files,config_files);
gen_dos_fd_gaus_fd();

//getchar();
//printf("about to run for dark\n");
mkdir("./sim", S_IRWXU | S_IRWXG | S_IROTH | S_IXOTH);

for (i=0;i<fitconfig->fit_names_len;i++)
{
	printf("Submitting job %d - %s\n",i,fitconfig->fit_names[i]);
	fit_build_job(myserver,fitconfig->fit_names[i]);
}

print_jobs(myserver);
error=0.0;

if (server_run_jobs(myserver)!=0)
{
	error+=100.0;
	printf("Error in newton solver detected by fit_run.c\n");
}else
{
error+=get_all_error(fitconfig);
}
change_cpus(myserver);


printf("Jobs/s=%le ODEs/s=%le\n",server_get_jobs_per_s(),server_get_odes_per_s());
#endif
return error;
}

