// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "../../sim.h"
#include "../../ntricks.h"
#include "li.h"

static int unused __attribute__((unused));

struct li li_config;

void sim_li_sc(struct device *in)
{
li_sc_load_config(&li_config,in);
//<clean>
int ittr=0;
double J;

struct istruct klist;
inter_init(&klist);

solve_all(in);

double sun=li_config.li_sc_sunstart;
//double sun_step=li_config.li_sc_sunstop;
FILE *out;
out=fopen("./li_sc.dat","w");
fclose(out);
	do
	{
		in->Psun=sun;
		light_solve_and_update(in,&(in->mylight),in->Psun,0.0,0.0);

		sim_externalv(in,0.0);
		J=get_equiv_J(in);

		ittr++;

		plot_now(in,"plot");

		#ifndef mindump
		printf("V=%lf %lf current = %e mA (%e A/m^2)\n",in->Vapplied,get_equiv_V(in),get_I(in)/1e-3,J);
		#endif

		if (in->stop_start==TRUE) getchar();

		sun+=li_config.li_sc_sundp;
		li_config.li_sc_sundp*=li_config.li_sc_sundpmul;
		printf("At sun intensity = %le\n",sun);

		out=fopen("./li_sc.dat","a");
		fprintf(out,"%le %le %le\n",sun,J,get_extracted_np(in));
		fclose(out);

		if (sun>li_config.li_sc_sunstop) in->stop=TRUE;

		if (in->stop==TRUE)
		{
			break;
		}

	}while(1);

inter_save_a(&klist,in->outputpath,"k.dat");
inter_free(&klist);
//</clean>
}

void sim_li(struct device *in)
{
//<clean>
int ittr=0;
double J;

struct istruct klist;
inter_init(&klist);

solve_all(in);

double sun=1.0;
double sun_step=10.0;

	do
	{
		in->Psun=sun;
		light_solve_and_update(in,&(in->mylight),in->Psun,0.0,0.0);

		sim_i(in,0.0);
		J=get_equiv_J(in);

		//add_avg_k_to_list(in,&klist);

		ittr++;

		plot_now(in,"plot");

		#ifndef mindump
		printf("V=%lf %lf current = %e mA (%e A/m^2)\n",in->Vapplied,get_equiv_V(in),get_I(in)/1e-3,J);
		#endif

		if (in->stop_start==TRUE) getchar();

		sun+=sun_step;
		printf("------------------%le--------------------------\n",sun);

		if (sun>1000.0)
		{
			in->stop=TRUE;
		}


		if (in->stop==TRUE)
		{
			break;

		}

	}while(1);

inter_save_a(&klist,in->outputpath,"k.dat");
inter_free(&klist);
//</clean>
}


void li_sc_load_config(struct li *in,struct device *dev)
{
char name[200];
FILE *config=fopena(dev->outputpath,"li_sc.inp","r");

unused=fscanf(config,"%s",name);
unused=fscanf(config,"%le",&(in->li_sc_sunstart));
unused=fscanf(config,"%s",name);
unused=fscanf(config,"%le",&(in->li_sc_sunstop));
unused=fscanf(config,"%s",name);
unused=fscanf(config,"%le",&(in->li_sc_sundp));
unused=fscanf(config,"%s",name);
unused=fscanf(config,"%le",&(in->li_sc_sundpmul));

unused=fscanf(config,"%s",name);
if (strcmp("#end",name)!=0)
{
printf("Problem with li_sc.inp file last item read %s!\n",name);
exit(0);
}
fclose(config);

}
